@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2015 HP Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,18 +26,19 @@ import os.path
 import struct
 import time
 import threading
-
+from base.sixext.moves import StringIO
+from io import BytesIO
 # Local
 from base.g import *
 from base.codes import *
 from base import device, utils, pml, codes
 from prnt import cups
-from fax import *
+from .fax import *
 
 
 # **************************************************************************** #
 
-# Page flags 
+# Page flags
 PAGE_FLAG_NONE = 0x00
 PAGE_FLAG_NEW_PAGE = 0x01
 PAGE_FLAG_END_PAGE = 0x02
@@ -69,12 +70,12 @@ DT_SPEED_DIALS   = 5
 DT_FAX_LOGS      = 6
 DT_CFG_PARMS     = 7
 DT_LANG_STRS     = 8
-DT_JUNK_FAX_CSIDS= 9  
-DT_REPORT_STRS   = 10  
+DT_JUNK_FAX_CSIDS= 9
+DT_REPORT_STRS   = 10
 DT_FONTS         = 11
 DT_TTI_BITMAP    = 12
 DT_COUNTERS      = 13
-DT_DEF_PARMS     = 14  
+DT_DEF_PARMS     = 14
 DT_SCAN_OPTIONS  = 15
 DT_FW_JOB_TABLE  = 17
 
@@ -97,7 +98,7 @@ RASTER_RECORD_SIZE = 4
 EOP_RECORD_SIZE = 12
 DIAL_STRING_RECORD_SIZE = 51
 
-# Page flags 
+# Page flags
 PAGE_FLAG_NEW_PAGE = 0x01
 PAGE_FLAG_END_PAGE = 0x02
 PAGE_FLAG_NEW_DOC = 0x04
@@ -124,39 +125,49 @@ RASTER_DATA_SIZE = 504
 class PMLFaxDevice(FaxDevice):
 
     def __init__(self, device_uri=None, printer_name=None,
-                 hpssd_sock=None, callback=None, 
-                 fax_type=FAX_TYPE_NONE):
+                 callback=None,
+                 fax_type=FAX_TYPE_NONE,
+                 disable_dbus=False):
 
-        FaxDevice.__init__(self, device_uri, 
-                           printer_name, hpssd_sock, 
-                           callback, fax_type)
+        FaxDevice.__init__(self, device_uri,
+                           printer_name,
+                           callback, fax_type,
+                           disable_dbus)
 
         self.send_fax_thread = None
         self.upload_log_thread = None
-        
+
 
     def setPhoneNum(self, num):
         return self.setPML(pml.OID_FAX_LOCAL_PHONE_NUM, str(num))
 
     def getPhoneNum(self):
-        return utils.printable(str(self.getPML(pml.OID_FAX_LOCAL_PHONE_NUM)[1]))
-
+        if PY3:
+            data = utils.printable(self.getPML(pml.OID_FAX_LOCAL_PHONE_NUM)[1].encode('utf-8'))
+            return data.decode('utf-8')
+        else:
+            return utils.printable(self.getPML(pml.OID_FAX_LOCAL_PHONE_NUM)[1])
     phone_num = property(getPhoneNum, setPhoneNum, doc="OID_FAX_LOCAL_PHONE_NUM")
 
 
     def setStationName(self, name):
-        return self.setPML(pml.OID_FAX_STATION_NAME, str(name))
+        return self.setPML(pml.OID_FAX_STATION_NAME, name)
 
     def getStationName(self):
-        return utils.printable(str(self.getPML(pml.OID_FAX_STATION_NAME)[1]))
+        if PY3:
+            data = utils.printable(self.getPML(pml.OID_FAX_STATION_NAME)[1].encode('utf-8'))
+            return data.decode('utf-8')
+        else:
+            return utils.printable(self.getPML(pml.OID_FAX_STATION_NAME)[1])
 
     station_name = property(getStationName, setStationName, doc="OID_FAX_STATION_NAME")
 
-    def setDateAndTime(self):
-        t = time.localtime()
-        p = struct.pack("BBBBBBB", t[0]-2000, t[1], t[2], t[6]+1, t[3], t[4], t[5])
-        log.debug(repr(p))
-        return self.setPML(pml.OID_DATE_AND_TIME, p)
+    def setDateAndTime(self):    #Need Revisit
+        pass
+        #t = time.localtime()
+        #p = struct.pack("BBBBBBB", t[0]-2000, t[1], t[2], t[6]+1, t[3], t[4], t[5])
+        #log.debug(repr(p))
+        #return self.setPML(pml.OID_DATE_AND_TIME, p.decode('latin-1'))
 
     def uploadLog(self):
         if not self.isUloadLogActive():
@@ -168,35 +179,35 @@ class PMLFaxDevice(FaxDevice):
 
     def isUploadLogActive(self):
         if self.upload_log_thread is not None:
-            return self.upload_log_thread.isAlive()
+            return self.upload_log_thread.is_alive()
         else:
             return False
 
     def waitForUploadLogThread(self):
         if self.upload_log_thread is not None and \
-            self.upload_log_thread.isAlive():
+            self.upload_log_thread.is_alive():
 
             self.upload_log_thread.join()
 
-    def sendFaxes(self, phone_num_list, fax_file_list, cover_message='', cover_re='', 
-                  cover_func=None, preserve_formatting=False, printer_name='', 
+    def sendFaxes(self, phone_num_list, fax_file_list, cover_message='', cover_re='',
+                  cover_func=None, preserve_formatting=False, printer_name='',
                   update_queue=None, event_queue=None):
 
         if not self.isSendFaxActive():
-    
-            self.send_fax_thread = PMLFaxSendThread(self, phone_num_list, fax_file_list, 
-                                                    cover_message, cover_re, cover_func, 
-                                                    preserve_formatting, 
-                                                    printer_name, update_queue, 
+
+            self.send_fax_thread = PMLFaxSendThread(self, self.service, phone_num_list, fax_file_list,
+                                                    cover_message, cover_re, cover_func,
+                                                    preserve_formatting,
+                                                    printer_name, update_queue,
                                                     event_queue)
-            
+
             self.send_fax_thread.start()
             return True
         else:
             return False
 
-            
-            
+
+
 # **************************************************************************** #
 class PMLUploadLogThread(threading.Thread):
     def __init__(self, dev):
@@ -219,7 +230,7 @@ class PMLUploadLogThread(threading.Thread):
         state = STATE_CHECK_IDLE
 
         while state != STATE_DONE: # --------------------------------- Log upload state machine
-            if state == STATE_ABORT: 
+            if state == STATE_ABORT:
                 pass
             elif state == STATE_SUCCESS:
                 pass
@@ -230,7 +241,7 @@ class PMLUploadLogThread(threading.Thread):
                 state = STATE_REQUEST_START
                 try:
                     self.dev.open()
-                except Error, e:
+                except Error as e:
                     log.error("Unable to open device (%s)." % e.msg)
                     state = STATE_ERROR
                 else:
@@ -282,14 +293,14 @@ class PMLUploadLogThread(threading.Thread):
                 self.dev.close()
 
 
-        
+
 # **************************************************************************** #
 class PMLFaxSendThread(FaxSendThread):
-    def __init__(self, dev, phone_num_list, fax_file_list, 
+    def __init__(self, dev, service, phone_num_list, fax_file_list,
                  cover_message='', cover_re='', cover_func=None, preserve_formatting=False,
                  printer_name='', update_queue=None, event_queue=None):
-                 
-        FaxSendThread.__init__(self, dev, phone_num_list, fax_file_list, 
+
+        FaxSendThread.__init__(self, dev, service, phone_num_list, fax_file_list,
              cover_message, cover_re, cover_func, preserve_formatting,
              printer_name, update_queue, event_queue)
 
@@ -354,7 +365,7 @@ class PMLFaxSendThread(FaxSendThread):
                 try:
                     try:
                         self.dev.open()
-                    except Error, e:
+                    except Error as e:
                         log.error("Unable to open device (%s)." % e.msg)
                         state = STATE_ERROR
                     else:
@@ -374,21 +385,24 @@ class PMLFaxSendThread(FaxSendThread):
             elif state == STATE_PRERENDER: # --------------------------------- Pre-render non-G3 files (40, 0, 0)
                 log.debug("%s State: Pre-render non-G3 files" % ("*"*20))
                 state = self.pre_render(STATE_COUNT_PAGES)
-                
+
 
             elif state == STATE_COUNT_PAGES: # --------------------------------- Get total page count (50, 0, 0)
                 log.debug("%s State: Get total page count" % ("*"*20))
                 state = self.count_pages(STATE_NEXT_RECIPIENT)
-                
+
 
             elif state == STATE_NEXT_RECIPIENT: # --------------------------------- Loop for multiple recipients (60, 0, 0)
                 log.debug("%s State: Next recipient" % ("*"*20))
                 state = STATE_COVER_PAGE
 
                 try:
-                    recipient = next_recipient.next()
+                    recipient = next(next_recipient)
                     #print recipient
                     log.debug("Processing for recipient %s" % recipient['name'])
+
+                    self.write_queue((STATUS_SENDING_TO_RECIPIENT, 0, recipient['name']))
+
                 except StopIteration:
                     state = STATE_SUCCESS
                     log.debug("Last recipient.")
@@ -435,6 +449,7 @@ class PMLFaxSendThread(FaxSendThread):
                 FAX_SEND_STATE_CLOSE_SESSION = 170
 
                 monitor_state = False
+                error_state = pml.DN_ERROR_NONE
                 fax_send_state = FAX_SEND_STATE_DEVICE_OPEN
 
                 while fax_send_state != FAX_SEND_STATE_DONE:
@@ -444,7 +459,7 @@ class PMLFaxSendThread(FaxSendThread):
                         fax_send_state = FAX_SEND_STATE_ABORT
 
                     if monitor_state:
-                        fax_state = self.getFaxDownloadState() 
+                        fax_state = self.getFaxDownloadState()
                         if not fax_state in (pml.UPDN_STATE_XFERACTIVE, pml.UPDN_STATE_XFERDONE):
                             log.error("D/L error state=%d" % fax_state)
                             fax_send_state = FAX_SEND_STATE_ERROR
@@ -460,6 +475,8 @@ class PMLFaxSendThread(FaxSendThread):
 
                     elif fax_send_state == FAX_SEND_STATE_ERROR: # -------------- Error (110, 20, 0)
                         log.error("Fax send error.")
+                        error_state = self.getFaxDownloadError()
+                        log.debug("Error State=%d (%s)" % (error_state, pml.DN_ERROR_STR.get(error_state, "Unknown")))
                         monitor_state = False
 
                         fax_send_state = FAX_SEND_STATE_RESET_TOKEN
@@ -482,7 +499,7 @@ class PMLFaxSendThread(FaxSendThread):
                         fax_send_state = FAX_SEND_STATE_SET_TOKEN
                         try:
                             self.dev.open()
-                        except Error, e:
+                        except Error as e:
                             log.error("Unable to open device (%s)." % e.msg)
                             fax_send_state = FAX_SEND_STATE_ERROR
                         else:
@@ -526,7 +543,7 @@ class PMLFaxSendThread(FaxSendThread):
                             log.debug("Opening fax channel.")
                             try:
                                 self.dev.openFax()
-                            except Error, e:
+                            except Error as e:
                                 log.error("Unable to open channel (%s)." % e.msg)
                                 fax_send_state = FAX_SEND_STATE_ERROR
                         else:
@@ -547,27 +564,30 @@ class PMLFaxSendThread(FaxSendThread):
                             (tx_status == pml.FAXJOB_TX_STATUS_IDLE or tx_status == pml.FAXJOB_TX_STATUS_DONE) and \
                             (rx_status == pml.FAXJOB_RX_STATUS_IDLE or rx_status == pml.FAXJOB_RX_STATUS_DONE)):
 
-                            if state == pml.UPDN_STATE_IDLE:
+                            # xwas if state == pml.UPDN_STATE_IDLE:
+                            if dl_state == pml.UPDN_STATE_IDLE:
                                 log.debug("Starting in idle state")
                             else:
-                                log.debug("Reseting to idle...")
+                                log.debug("Resetting to idle...")
                                 self.dev.setPML(pml.OID_FAX_DOWNLOAD, pml.UPDN_STATE_IDLE)
                                 time.sleep(0.5)
                         else:
                             fax_send_state = FAX_SEND_STATE_BUSY
 
-                    elif fax_send_state == FAX_SEND_STATE_START_REQUEST: # -------------- Request fax start (110, 90, 0) 
+                    elif fax_send_state == FAX_SEND_STATE_START_REQUEST: # -------------- Request fax start (110, 90, 0)
                         log.debug("%s State: Request start" % ("*"*20))
                         fax_send_state = FAX_SEND_STATE_SET_PARAMS
 
                         dl_state = self.getFaxDownloadState()
 
                         if dl_state == pml.UPDN_STATE_IDLE:
+                            log.debug("Try: 0")
+                            log.debug("Setting to up/down state request start...")
                             self.dev.setPML(pml.OID_FAX_DOWNLOAD, pml.UPDN_STATE_REQSTART)
                             time.sleep(1)
 
                             log.debug("Waiting for active state...")
-                            i = 0
+                            i = 1
 
                             while i < 10:
                                 log.debug("Try: %d" % i)
@@ -582,11 +602,12 @@ class PMLFaxSendThread(FaxSendThread):
                                     break
 
                                 time.sleep(1)
+                                log.debug("Setting to up/down state request start...")
                                 self.dev.setPML(pml.OID_FAX_DOWNLOAD, pml.UPDN_STATE_REQSTART)
 
                                 i += 1
 
-                            else:  
+                            else:
                                 log.error("Could not get into active state!")
                                 fax_send_state = FAX_SEND_STATE_BUSY
 
@@ -604,9 +625,9 @@ class PMLFaxSendThread(FaxSendThread):
                         try:
                             self.dev.setPML(pml.OID_DEV_DOWNLOAD_TIMEOUT, pml.DEFAULT_DOWNLOAD_TIMEOUT)
                             self.dev.setPML(pml.OID_FAXJOB_TX_TYPE, pml.FAXJOB_TX_TYPE_HOST_ONLY)
-                            log.debug("Setting date and time on device.")                            
-                            self.dev.setDateAndTime() 
-                        except Error, e:
+                            log.debug("Setting date and time on device.")
+                            self.dev.setDateAndTime()
+                        except Error as e:
                             log.error("PML/SNMP error (%s)" % e.msg)
                             fax_send_state = FAX_SEND_STATE_ERROR
 
@@ -633,7 +654,7 @@ class PMLFaxSendThread(FaxSendThread):
                         log.debug("Dialing: %s" % recipient['fax'])
 
                         log.debug("Sending dial strings...")
-                        self.create_mfpdtf_fixed_header(DT_DIAL_STRINGS, True, 
+                        self.create_mfpdtf_fixed_header(DT_DIAL_STRINGS, True,
                             PAGE_FLAG_NEW_DOC | PAGE_FLAG_END_DOC | PAGE_FLAG_END_STREAM) # 0x1c on Windows, we were sending 0x0c
                         #print recipient
                         dial_strings = recipient['fax'].encode('ascii')
@@ -652,7 +673,7 @@ class PMLFaxSendThread(FaxSendThread):
                         fax_send_state = FAX_SEND_STATE_SEND_PAGES
 
                         try:
-                            ff = file(self.f, 'r')
+                            ff = open(self.f, 'rb')
                         except IOError:
                             log.error("Unable to read fax file.")
                             fax_send_state = FAX_SEND_STATE_ERROR
@@ -668,8 +689,7 @@ class PMLFaxSendThread(FaxSendThread):
                         magic, version, total_pages, hort_dpi, vert_dpi, page_size, \
                             resolution, encoding, reserved1, reserved2 = self.decode_fax_header(header)
 
-
-                        if magic != 'hplip_g3':
+                        if magic != b'hplip_g3':
                             log.error("Invalid file header. Bad magic.")
                             fax_send_state = FAX_SEND_STATE_ERROR
                         else:
@@ -690,7 +710,7 @@ class PMLFaxSendThread(FaxSendThread):
                     elif fax_send_state == FAX_SEND_STATE_SEND_PAGES:  # --------------------------------- Send fax pages state machine (110, 130, 0)
                         log.debug("%s State: Send pages" % ("*"*20))
                         fax_send_state = FAX_SEND_STATE_SEND_END_OF_STREAM
-                        page = StringIO()
+                        page = BytesIO()
 
                         for p in range(total_pages):
 
@@ -739,9 +759,16 @@ class PMLFaxSendThread(FaxSendThread):
                                 data = page.read(RASTER_DATA_SIZE)
                                 total_read += RASTER_DATA_SIZE
 
-                                self.getFaxDownloadState()
+                                dl_state = self.getFaxDownloadState()
+                                if dl_state == pml.UPDN_STATE_ERRORABORT:
+                                    fax_send_state = FAX_SEND_STATE_ERROR
+                                    break
 
-                                if data == '':
+                                if self.check_for_cancel():
+                                    fax_send_state = FAX_SEND_STATE_ABORT
+                                    break
+
+                                if data == b'':
                                     self.create_eop_record(rpp)
 
                                     try:
@@ -760,7 +787,7 @@ class PMLFaxSendThread(FaxSendThread):
                                         break
 
                                 status = self.getFaxJobTxStatus()
-                                while status  == pml.FAXJOB_TX_STATUS_DIALING:
+                                while status == pml.FAXJOB_TX_STATUS_DIALING:
                                     self.write_queue((STATUS_DIALING, 0, recipient['fax']))
                                     time.sleep(1.0)
 
@@ -777,7 +804,7 @@ class PMLFaxSendThread(FaxSendThread):
 
                                 if fax_send_state not in (FAX_SEND_STATE_ABORT, FAX_SEND_STATE_ERROR):
 
-                                    while status  == pml.FAXJOB_TX_STATUS_CONNECTING: 
+                                    while status == pml.FAXJOB_TX_STATUS_CONNECTING:
                                         self.write_queue((STATUS_CONNECTING, 0, recipient['fax']))
                                         time.sleep(1.0)
 
@@ -792,7 +819,7 @@ class PMLFaxSendThread(FaxSendThread):
 
                                         status = self.getFaxJobTxStatus()
 
-                                if status == pml.FAXJOB_TX_STATUS_TRANSMITTING:    
+                                if status == pml.FAXJOB_TX_STATUS_TRANSMITTING:
                                     self.write_queue((STATUS_SENDING, page_num, recipient['fax']))
 
                                 self.create_mfpdtf_fixed_header(DT_FAX_IMAGES, page_flags=0)
@@ -802,7 +829,7 @@ class PMLFaxSendThread(FaxSendThread):
                                     break
 
                             page.truncate(0)
-                            page.seek(0)                
+                            page.seek(0)
 
 
                     elif fax_send_state == FAX_SEND_STATE_SEND_END_OF_STREAM: # -------------- EOS (110, 140, 0)
@@ -830,21 +857,25 @@ class PMLFaxSendThread(FaxSendThread):
 
                         if status == pml.FAXJOB_TX_STATUS_DIALING:
                                 self.write_queue((STATUS_DIALING, 0, recipient['fax']))
+                                log.debug("Dialing ...")
 
-                        elif status == pml.FAXJOB_TX_STATUS_TRANSMITTING:    
+                        elif status == pml.FAXJOB_TX_STATUS_TRANSMITTING:
                             self.write_queue((STATUS_SENDING, page_num, recipient['fax']))
+                            log.debug("Transmitting ...")
 
                         elif status in (pml.FAXJOB_TX_STATUS_DONE, pml.FAXJOB_RX_STATUS_IDLE):
                             fax_send_state = FAX_SEND_STATE_RESET_TOKEN
                             state = STATE_NEXT_RECIPIENT
+                            log.debug("Transmitting done or idle ...")
 
                         else:
                             self.write_queue((STATUS_SENDING, page_num, recipient['fax']))
+                            log.debug("Pending ...")
 
 
                     elif fax_send_state == FAX_SEND_STATE_RESET_TOKEN: # -------------- Release fax token (110, 160, 0)
-                        self.write_queue((STATUS_CLEANUP, 0, ''))
                         log.debug("%s State: Release fax token" % ("*"*20))
+                        self.write_queue((STATUS_CLEANUP, 0, ''))
 
                         try:
                             self.dev.setPML(pml.OID_FAX_TOKEN, '\x00'*16)
@@ -910,6 +941,13 @@ class PMLFaxSendThread(FaxSendThread):
         else:
             return pml.UPDN_STATE_ERRORABORT
 
+    def getFaxDownloadError(self):
+        result_code, state = self.dev.getPML(pml.OID_FAX_DOWNLOAD_ERROR)
+        if state:
+            return state
+        else:
+            return pml.DN_ERROR_UNKNOWN
+
     def getFaxJobTxStatus(self):
         result_code, status = self.dev.getPML(pml.OID_FAXJOB_TX_STATUS)
         if status:
@@ -944,12 +982,12 @@ class PMLFaxSendThread(FaxSendThread):
             elif data_type == DT_FAX_IMAGES:
                 header_len += FAX_IMAGE_VARIANT_HEADER_SIZE
 
-        self.stream.write(struct.pack("<IHBB", 
+        self.stream.write(struct.pack("<IHBB",
                           0, header_len, data_type, page_flags))
 
 
     def create_mfpdtf_dial_strings(self, number):
-        p = struct.pack("<BBHH51s", 
+        p = struct.pack("<BBHH51s",
                           MAJOR_VER, MINOR_VER,
                           1, 51, number[:51])
         log.debug(repr(p))
@@ -973,7 +1011,7 @@ class PMLFaxSendThread(FaxSendThread):
 
     def create_eop_record(self, rpp):
         self.stream.write(struct.pack("<BBBBII",
-                            RT_END_PAGE, 0, 0, 0, 
+                            RT_END_PAGE, 0, 0, 0,
                             rpp, 0,))
 
 
@@ -985,31 +1023,13 @@ class PMLFaxSendThread(FaxSendThread):
 
 
     def create_mfpdtf_fax_header(self, total_pages):
-        self.stream.write(struct.pack("<BBBHBI20s20s20sI", 
-                            MAJOR_VER, MINOR_VER, SRC_HOST, total_pages, 
-                            TTI_PREPENDED_TO_IMAGE, 0, '', '', '', 0))
+        self.stream.write(struct.pack("<BBBHBI20s20s20sI",
+                            MAJOR_VER, MINOR_VER, SRC_HOST, total_pages,
+                            TTI_PREPENDED_TO_IMAGE, 0, b'', b'', b'', 0))
 
 
     def write_stream(self):
-        self.adjust_fixed_header_block_size() 
+        self.adjust_fixed_header_block_size()
         self.dev.writeFax(self.stream.getvalue())
         self.stream.truncate(0)
-        self.stream.seek(0)    
-
-
-    def decode_fax_header(self, header):
-        try:
-            return struct.unpack(">8sBIHHBBBII", header)
-        except struct.error:
-            return -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
-
-    def decode_page_header(self, header):
-        try:
-            return struct.unpack(">IIIIII", header)
-        except struct.error:
-            return -1, -1, -1, -1, -1, -1
-
-            
-            
-            
-  
+        self.stream.seek(0)
